@@ -1,6 +1,6 @@
 // -*- mode: cpp; mode: fold -*-
 // Description								/*{{{*/
-// $Id: pkgcachegen.cc,v 1.50 2001/07/01 22:28:24 jgg Exp $
+// $Id: pkgcachegen.cc,v 1.53 2003/02/02 02:44:20 doogie Exp $
 /* ######################################################################
    
    Package Cache Generator - Generator for the cache structure.
@@ -42,7 +42,8 @@ typedef vector<pkgIndexFile *>::iterator FileIterator;
 // ---------------------------------------------------------------------
 /* We set the diry flag and make sure that is written to the disk */
 pkgCacheGenerator::pkgCacheGenerator(DynamicMMap *pMap,OpProgress *Prog) :
-                    Map(*pMap), Cache(pMap,false), Progress(Prog)
+		    Map(*pMap), Cache(pMap,false), Progress(Prog),
+		    FoundFileDeps(0)
 {
    CurrentFile = 0;
    memset(UniqHash,0,sizeof(UniqHash));
@@ -125,7 +126,8 @@ bool pkgCacheGenerator::MergeList(ListParser &List,
       if (Version.empty() == true)
       {
 	 if (List.UsePackage(Pkg,pkgCache::VerIterator(Cache)) == false)
-	    return _error->Error(_("Error occured while processing %s (UsePackage1)"),PackageName.c_str());
+	    return _error->Error(_("Error occured while processing %s (UsePackage1)"),
+				 PackageName.c_str());
 	 continue;
       }
 
@@ -145,15 +147,18 @@ bool pkgCacheGenerator::MergeList(ListParser &List,
       if (Res == 0 && Ver->Hash == Hash)
       {
 	 if (List.UsePackage(Pkg,Ver) == false)
-	    return _error->Error(_("Error occured while processing %s (UsePackage2)"),PackageName.c_str());
+	    return _error->Error(_("Error occured while processing %s (UsePackage2)"),
+				 PackageName.c_str());
 
 	 if (NewFileVer(Ver,List) == false)
-	    return _error->Error(_("Error occured while processing %s (NewFileVer1)"),PackageName.c_str());
+	    return _error->Error(_("Error occured while processing %s (NewFileVer1)"),
+				 PackageName.c_str());
 	 
 	 // Read only a single record and return
 	 if (OutVer != 0)
 	 {
 	    *OutVer = Ver;
+	    FoundFileDeps |= List.HasFileDeps();
 	    return true;
 	 }
 	 
@@ -176,22 +181,28 @@ bool pkgCacheGenerator::MergeList(ListParser &List,
       Ver->ParentPkg = Pkg.Index();
       Ver->Hash = Hash;
       if (List.NewVersion(Ver) == false)
-	 return _error->Error(_("Error occured while processing %s (NewVersion1)"),PackageName.c_str());
+	 return _error->Error(_("Error occured while processing %s (NewVersion1)"),
+			      PackageName.c_str());
 
       if (List.UsePackage(Pkg,Ver) == false)
-	 return _error->Error(_("Error occured while processing %s (UsePackage3)"),PackageName.c_str());
+	 return _error->Error(_("Error occured while processing %s (UsePackage3)"),
+			      PackageName.c_str());
       
       if (NewFileVer(Ver,List) == false)
-	 return _error->Error(_("Error occured while processing %s (NewVersion2)"),PackageName.c_str());
+	 return _error->Error(_("Error occured while processing %s (NewVersion2)"),
+			      PackageName.c_str());
 
       // Read only a single record and return
       if (OutVer != 0)
       {
 	 *OutVer = Ver;
+	 FoundFileDeps |= List.HasFileDeps();
 	 return true;
       }      
    }
 
+   FoundFileDeps |= List.HasFileDeps();
+
    if (Cache.HeaderP->PackageCount >= (1ULL<<sizeof(Cache.PkgP->ID)*8)-1)
       return _error->Error(_("Wow, you exceeded the number of package "
 			     "names this APT is capable of."));
@@ -204,6 +215,54 @@ bool pkgCacheGenerator::MergeList(ListParser &List,
    return true;
 }
 									/*}}}*/
+// CacheGenerator::MergeFileProvides - Merge file provides   		/*{{{*/
+// ---------------------------------------------------------------------
+/* If we found any file depends while parsing the main list we need to 
+   resolve them. Since it is undesired to load the entire list of files
+   into the cache as virtual packages we do a two stage effort. MergeList
+   identifies the file depends and this creates Provdies for them by
+   re-parsing all the indexs. */
+bool pkgCacheGenerator::MergeFileProvides(ListParser &List)
+{
+   List.Owner = this;
+   
+   unsigned int Counter = 0;
+   while (List.Step() == true)
+   {
+      string PackageName = List.Package();
+      if (PackageName.empty() == true)
+	 return false;
+      string Version = List.Version();
+      if (Version.empty() == true)
+	 continue;
+      
+      pkgCache::PkgIterator Pkg = Cache.FindPkg(PackageName);
+      if (Pkg.end() == true)
+	 return _error->Error(_("Error occured while processing %s (FindPkg)"),
+				PackageName.c_str());
+      Counter++;
+      if (Counter % 100 == 0 && Progress != 0)
+	 Progress->Progress(List.Offset());
+
+      unsigned long Hash = List.VersionHash();
+      pkgCache::VerIterator Ver = Pkg.VersionList();
+      for (; Ver.end() == false; Ver++)
+      {
+	 if (Ver->Hash == Hash && Version.c_str() == Ver.VerStr())
+	 {
+	    if (List.CollectFileProvides(Cache,Ver) == false)
+	       return _error->Error(_("Error occured while processing %s (CollectFileProvides)"),PackageName.c_str());
+	    break;
+	 }
+      }
+      
+      if (Ver.end() == true)
+	 _error->Warning(_("Package %s %s was not found while processing file dependencies"),PackageName.c_str(),Version.c_str());
+   }
+
+   return true;
+}
+									/*}}}*/
 // CacheGenerator::NewPackage - Add a new package			/*{{{*/
 // ---------------------------------------------------------------------
 /* This creates a new package structure and adds it to the hash table */
@@ -344,6 +403,10 @@ bool pkgCacheGenerator::ListParser::NewDepends(pkgCache::VerIterator Ver,
 	 OldDepLast = &D->NextDepends;
       OldDepVer = Ver;
    }
+
+   // Is it a file dependency?
+   if (PackageName[0] == '/')
+      FoundFileDeps = true;
    
    Dep->NextDepends = *OldDepLast;
    *OldDepLast = Dep.Index();
@@ -560,28 +623,44 @@ static bool BuildCache(pkgCacheGenerator &Gen,
 		       unsigned long &CurrentSize,unsigned long TotalSize,
 		       FileIterator Start, FileIterator End)
 {
-   for (; Start != End; Start++)
+   FileIterator I;
+   for (I = Start; I != End; I++)
    {
-      if ((*Start)->HasPackages() == false)
+      if ((*I)->HasPackages() == false)
 	 continue;
       
-      if ((*Start)->Exists() == false)
+      if ((*I)->Exists() == false)
 	 continue;
 
-      if ((*Start)->FindInCache(Gen.GetCache()).end() == false)
+      if ((*I)->FindInCache(Gen.GetCache()).end() == false)
       {
 	 _error->Warning("Duplicate sources.list entry %s",
-			 (*Start)->Describe().c_str());
+			 (*I)->Describe().c_str());
 	 continue;
       }
       
-      unsigned long Size = (*Start)->Size();
+      unsigned long Size = (*I)->Size();
       Progress.OverallProgress(CurrentSize,TotalSize,Size,_("Reading Package Lists"));
       CurrentSize += Size;
       
-      if ((*Start)->Merge(Gen,Progress) == false)
+      if ((*I)->Merge(Gen,Progress) == false)
 	 return false;
    }   
+
+   if (Gen.HasFileDeps() == true)
+   {
+      Progress.Done();
+      TotalSize = ComputeSize(Start, End);
+      CurrentSize = 0;
+      for (I = Start; I != End; I++)
+      {
+	 unsigned long Size = (*I)->Size();
+	 Progress.OverallProgress(CurrentSize,TotalSize,Size,_("Collecting File Provides"));
+	 CurrentSize += Size;
+	 if ((*I)->MergeFileProvides(Gen,Progress) == false)
+	    return false;
+      }
+   }
    
    return true;
 }
@@ -597,7 +676,7 @@ static bool BuildCache(pkgCacheGenerator &Gen,
 bool pkgMakeStatusCache(pkgSourceList &List,OpProgress &Progress,
 			MMap **OutMap,bool AllowMem)
 {
-   unsigned long MapSize = _config->FindI("APT::Cache-Limit",6*1024*1024);
+   unsigned long MapSize = _config->FindI("APT::Cache-Limit",12*1024*1024);
    
    vector<pkgIndexFile *> Files(List.begin(),List.end());
    unsigned long EndOfSource = Files.size();
@@ -636,6 +715,7 @@ bool pkgMakeStatusCache(pkgSourceList &List,OpProgress &Progress,
    {
       unlink(CacheFile.c_str());
       CacheF = new FileFd(CacheFile,FileFd::WriteEmpty);
+      fchmod(CacheF->Fd(),0644);
       Map = new DynamicMMap(*CacheF,MMap::Public,MapSize);
       if (_error->PendingError() == true)
 	 return false;
@@ -686,6 +766,9 @@ bool pkgMakeStatusCache(pkgSourceList &List,OpProgress &Progress,
 	 FileFd SCacheF(SrcCacheFile,FileFd::WriteEmpty);
 	 if (_error->PendingError() == true)
 	    return false;
+	 
+	 fchmod(SCacheF.Fd(),0644);
+	 
 	 // Write out the main data
 	 if (SCacheF.Write(Map->Data(),Map->Size()) == false)
 	    return _error->Error(_("IO Error saving source cache"));
@@ -696,8 +779,8 @@ bool pkgMakeStatusCache(pkgSourceList &List,OpProgress &Progress,
 	 if (SCacheF.Seek(0) == false ||
 	     SCacheF.Write(Map->Data(),sizeof(*Gen.GetCache().HeaderP)) == false)
 	    return _error->Error(_("IO Error saving source cache"));
-	 SCacheF.Sync();
 	 Gen.GetCache().HeaderP->Dirty = true;
+	 SCacheF.Sync();
       }
       
       // Build the status cache
@@ -729,7 +812,7 @@ bool pkgMakeStatusCache(pkgSourceList &List,OpProgress &Progress,
 /* */
 bool pkgMakeOnlyStatusCache(OpProgress &Progress,DynamicMMap **OutMap)
 {
-   unsigned long MapSize = _config->FindI("APT::Cache-Limit",4*1024*1024);
+   unsigned long MapSize = _config->FindI("APT::Cache-Limit",8*1024*1024);
    vector<pkgIndexFile *> Files;
    unsigned long EndOfSource = Files.size();
    if (_system->AddStatusFiles(Files) == false)
