@@ -1,6 +1,6 @@
 // -*- mode: cpp; mode: fold -*-
 // Description								/*{{{*/
-// $Id: tagfile.cc,v 1.30 2001/05/14 05:56:26 jgg Exp $
+// $Id: tagfile.cc,v 1.37 2003/05/19 17:13:57 doogie Exp $
 /* ######################################################################
 
    Fast scanner for RFC-822 type header information
@@ -68,13 +68,13 @@ bool pkgTagFile::Step(pkgTagSection &Tag)
 	 return false;
       
       if (Tag.Scan(Start,End - Start) == false)
-	 return _error->Error(_("Unable to parse package file %s (1)"),Fd.Name().c_str());
-   }   
+	 return _error->Error(_("Unable to parse package file %s (1)"),
+			      Fd.Name().c_str());
+   }
    Start += Tag.size();
    iOffset += Tag.size();
 
    Tag.Trim();
-   
    return true;
 }
 									/*}}}*/
@@ -85,14 +85,25 @@ bool pkgTagFile::Step(pkgTagSection &Tag)
 bool pkgTagFile::Fill()
 {
    unsigned long EndSize = End - Start;
+   unsigned long Actual = 0;
    
    memmove(Buffer,Start,EndSize);
    Start = Buffer;
    End = Buffer + EndSize;
    
+   if (Done == false)
+   {
+      // See if only a bit of the file is left
+      if (Fd.Read(End,Size - (End - Buffer),&Actual) == false)
+	 return false;
+      if (Actual != Size - (End - Buffer))
+	 Done = true;
+      End += Actual;
+   }
+   
    if (Done == true)
    {
-      if (EndSize <= 3)
+      if (EndSize <= 3 && Actual == 0)
 	 return false;
       if (Size - (End - Buffer) < 4)
 	 return true;
@@ -108,31 +119,6 @@ bool pkgTagFile::Fill()
       return true;
    }
    
-   // See if only a bit of the file is left
-   unsigned long Actual;
-   if (Fd.Read(End,Size - (End - Buffer),&Actual) == false)
-      return false;
-   if (Actual != Size - (End - Buffer))
-      Done = true;
-   End += Actual;
-/*   
-   if (Left < Size - (End - Buffer))
-   {
-      if (Fd.Read(End,Left) == false)
-	 return false;
-      
-      End += Left;
-      Left = 0;
-   }
-   else
-   {
-      if (Fd.Read(End,Size - (End - Buffer)) == false)
-	 return false;
-      
-      Left -= Size - (End - Buffer);
-      End = Buffer + Size;
-   }*/
-   
    return true;
 }
 									/*}}}*/
@@ -197,7 +183,7 @@ bool pkgTagSection::Scan(const char *Start,unsigned long MaxLength)
       return false;
    
    TagCount = 0;
-   while (TagCount < sizeof(Indexes)/sizeof(Indexes[0]) && Stop < End)
+   while (TagCount+1 < sizeof(Indexes)/sizeof(Indexes[0]) && Stop < End)
    {
       // Start a new index and add it to the hash
       if (isspace(Stop[0]) == 0)
@@ -211,13 +197,13 @@ bool pkgTagSection::Scan(const char *Start,unsigned long MaxLength)
       if (Stop == 0)
 	 return false;
       
-      for (; Stop[1] == '\r' && Stop+1 < End; Stop++);
+      for (; Stop+1 < End && Stop[1] == '\r'; Stop++);
 
       // Double newline marks the end of the record
       if (Stop+1 < End && Stop[1] == '\n')
       {
 	 Indexes[TagCount] = Stop - Section;
-	 for (; (Stop[0] == '\n' || Stop[0] == '\r') && Stop < End; Stop++);
+	 for (; Stop < End && (Stop[0] == '\n' || Stop[0] == '\r'); Stop++);
 	 return true;
       }
       
