@@ -1,6 +1,6 @@
 // -*- mode: cpp; mode: fold -*-
 // Description								/*{{{*/
-// $Id: acqprogress.cc,v 1.22 2001/05/27 04:46:43 jgg Exp $
+// $Id: acqprogress.cc,v 1.24 2003/04/27 01:56:48 doogie Exp $
 /* ######################################################################
 
    Acquire Progress - Command line progress meter 
@@ -152,7 +152,7 @@ bool AcqTextStatus::Pulse(pkgAcquire *Owner)
    
    enum {Long = 0,Medium,Short} Mode = Long;
    
-   char Buffer[1024];
+   char Buffer[sizeof(BlankLine)];
    char *End = Buffer + sizeof(Buffer);
    char *S = Buffer;
    if (ScreenWidth >= sizeof(Buffer))
@@ -268,8 +268,9 @@ bool AcqTextStatus::MediaChange(string Media,string Drive)
 {
    if (Quiet <= 0)
       cout << '\r' << BlankLine << '\r';
-   ioprintf(cout,_("Media Change: Please insert the disc labeled '%s' in "
-		   "the drive '%s' and press enter\n"),
+   ioprintf(cout,_("Media Change: Please insert the disc labeled\n"
+		   " '%s'\n"
+		   "in the drive '%s' and press enter\n"),
 	    Media.c_str(),Drive.c_str());
 
    char C = 0;
