@@ -752,7 +752,7 @@ int ShowHelp()
       "  -f   Fast mode, don't check package files\n"
       "  -a   Thorough scan mode\n"
       "  -c=? Read this configuration file\n"
-      "  -o=? Set an arbitary configuration option, eg -o dir::cache=/tmp\n"
+      "  -o=? Set an arbitrary configuration option, eg -o dir::cache=/tmp\n"
       "See fstab(5)\n";
    return 0;
 }
