@@ -1,10 +1,13 @@
 // -*- mode: cpp; mode: fold -*-
 // Description								/*{{{*/
-// $Id: connect.cc,v 1.7 2001/02/20 07:03:18 jgg Exp $
+// $Id: connect.cc,v 1.11 2004/01/07 20:39:38 mdz Exp $
 /* ######################################################################
 
    Connect - Replacement connect call
-   
+
+   This was originally authored by Jason Gunthorpe <jgg@debian.org>
+   and is placed in the Public Domain, do with it what you will.
+      
    ##################################################################### */
 									/*}}}*/
 // Include Files							/*{{{*/
@@ -23,6 +26,7 @@
 #include <netdb.h>
 
 #include "rfc2553emu.h"
+#include <apti18n.h>
 									/*}}}*/
 
 static string LastHost;
@@ -57,14 +61,14 @@ static bool DoConnect(struct addrinfo *Addr,string Host,
    getnameinfo(Addr->ai_addr,Addr->ai_addrlen,
 	       Name,sizeof(Name),Service,sizeof(Service),
 	       NI_NUMERICHOST|NI_NUMERICSERV);
-   Owner->Status("Connecting to %s (%s)",Host.c_str(),Name);
+   Owner->Status(_("Connecting to %s (%s)"),Host.c_str(),Name);
 
    /* If this is an IP rotation store the IP we are using.. If something goes
       wrong this will get tacked onto the end of the error message */
    if (LastHostAddr->ai_next != 0)
    {
       char Name2[NI_MAXHOST + NI_MAXSERV + 10];
-      snprintf(Name2,sizeof(Name2),"[IP: %s %s]",Name,Service);
+      snprintf(Name2,sizeof(Name2),_("[IP: %s %s]"),Name,Service);
       Owner->SetFailExtraMsg(string(Name2));
    }   
    else
@@ -73,31 +77,31 @@ static bool DoConnect(struct addrinfo *Addr,string Host,
    // Get a socket
    if ((Fd = socket(Addr->ai_family,Addr->ai_socktype,
 		    Addr->ai_protocol)) < 0)
-      return _error->Errno("socket","Could not create a socket for %s (f=%u t=%u p=%u)",
+      return _error->Errno("socket",_("Could not create a socket for %s (f=%u t=%u p=%u)"),
 			   Name,Addr->ai_family,Addr->ai_socktype,Addr->ai_protocol);
    
    SetNonBlock(Fd,true);
    if (connect(Fd,Addr->ai_addr,Addr->ai_addrlen) < 0 &&
        errno != EINPROGRESS)
-      return _error->Errno("connect","Cannot initiate the connection "
-			   "to %s:%s (%s).",Host.c_str(),Service,Name);
+      return _error->Errno("connect",_("Cannot initiate the connection "
+			   "to %s:%s (%s)."),Host.c_str(),Service,Name);
    
    /* This implements a timeout for connect by opening the connection
       nonblocking */
    if (WaitFd(Fd,true,TimeOut) == false)
-      return _error->Error("Could not connect to %s:%s (%s), "
-			   "connection timed out",Host.c_str(),Service,Name);
+      return _error->Error(_("Could not connect to %s:%s (%s), "
+			   "connection timed out"),Host.c_str(),Service,Name);
 
    // Check the socket for an error condition
    unsigned int Err;
    unsigned int Len = sizeof(Err);
    if (getsockopt(Fd,SOL_SOCKET,SO_ERROR,&Err,&Len) != 0)
-      return _error->Errno("getsockopt","Failed");
+      return _error->Errno("getsockopt",_("Failed"));
    
    if (Err != 0)
    {
       errno = Err;
-      return _error->Errno("connect","Could not connect to %s:%s (%s).",Host.c_str(),
+      return _error->Errno("connect",_("Could not connect to %s:%s (%s)."),Host.c_str(),
 			   Service,Name);
    }
    
@@ -125,7 +129,7 @@ bool Connect(string Host,int Port,const char *Service,int DefPort,int &Fd,
       sensible */
    if (LastHost != Host || LastPort != Port)
    {
-      Owner->Status("Connecting to %s",Host.c_str());
+      Owner->Status(_("Connecting to %s"),Host.c_str());
 
       // Free the old address structure
       if (LastHostAddr != 0)
@@ -156,10 +160,13 @@ bool Connect(string Host,int Port,const char *Service,int DefPort,int &Fd,
 		  DefPort = 0;
 		  continue;
 	       }
-	       return _error->Error("Could not resolve '%s'",Host.c_str());
+	       return _error->Error(_("Could not resolve '%s'"),Host.c_str());
 	    }
 	    
-	    return _error->Error("Something wicked happened resolving '%s:%s' (%i)",
+	    if (Res == EAI_AGAIN)
+	       return _error->Error(_("Temporary failure resolving '%s'"),
+				    Host.c_str());
+	    return _error->Error(_("Something wicked happened resolving '%s:%s' (%i)"),
 				 Host.c_str(),ServStr,Res);
 	 }
 	 break;
@@ -206,6 +213,6 @@ bool Connect(string Host,int Port,const char *Service,int DefPort,int &Fd,
 
    if (_error->PendingError() == true)
       return false;   
-   return _error->Error("Unable to connect to %s %s:",Host.c_str(),ServStr);
+   return _error->Error(_("Unable to connect to %s %s:"),Host.c_str(),ServStr);
 }
 									/*}}}*/
